'use strict'
var KBucket = require('../index')

var test = module.exports = {}

test['throws TypeError if contact.id is not a Buffer'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: 'foo' }
  test.throws(function () {
    kBucket.add(contact)
  }, TypeError)
  test.done()
}

test['adding a contact places it in bucket'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a') }
  kBucket.add(contact)
  test.ok(kBucket.bucket[0] === contact)
  test.done()
}

test['adding an existing contact does not increase number of contacts in bucket'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a') }
  kBucket.add(contact)
  kBucket.add({id: new Buffer('a')})
  test.equal(kBucket.bucket.length, 1)
  test.done()
}

test['adding same contact moves it to the end of the bucket (most-recently-contacted end)'] = function (test) {
  test.expect(5)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a') }
  kBucket.add(contact)
  test.equal(kBucket.bucket.length, 1)
  kBucket.add({ id: new Buffer('b') })
  test.equal(kBucket.bucket.length, 2)
  test.equal(kBucket.bucket[0], contact) // least-recently-contacted end
  kBucket.add(contact)
  test.equal(kBucket.bucket.length, 2)
  test.equal(kBucket.bucket[1], contact) // most-recently-contacted end
  test.done()
}

test['adding contact to bucket that can\'t be split results in calling "ping" callback'] = function (test) {
  test.expect(3 /* numberOfNodesToPing*/ + 2)
  var kBucket = new KBucket({ localNodeId: new Buffer([ 0x00, 0x00 ]) })
  kBucket.on('ping', function (contacts, replacement) {
    test.equal(contacts.length, kBucket.numberOfNodesToPing)
    // console.dir(kBucket.high.bucket[0])
    for (var i = 0; i < kBucket.numberOfNodesToPing; ++i) {
      // the least recently contacted end of the bucket should be pinged
      test.equal(contacts[i], kBucket.high.bucket[i])
    }
    test.deepEqual(replacement, { id: new Buffer([ 0x80, j ]) })
    test.done()
  })
  for (var j = 0; j < kBucket.numberOfNodesPerKBucket + 1; ++j) {
    kBucket.add({ id: new Buffer([ 0x80, j ]) }) // make sure all go into "far away" bucket
  }
}

test['should generate event "added" once'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a') }
  kBucket.on('added', function (newContact) {
    test.deepEqual(newContact, contact)
  })
  kBucket.add(contact)
  kBucket.add(contact)
  test.done()
}

test['should generate event "added" when adding to a split bucket'] = function (test) {
  test.expect(2)
  var kBucket = new KBucket({
    localNodeId: new Buffer('') // need non-random localNodeId for deterministic splits
  })
  for (var i = 0; i < kBucket.numberOfNodesPerKBucket + 1; ++i) {
    kBucket.add({ id: new Buffer('' + i) })
  }
  test.ok(!kBucket.bucket)
  var contact = { id: new Buffer('a') }
  kBucket.on('added', function (newContact) {
    test.deepEqual(newContact, contact)
  })
  kBucket.add(contact)
  test.done()
}
