'use strict'
var bufferEqual = require('buffer-equal')
var KBucket = require('../index')

var test = module.exports = {}

test['invalid index results in exception'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a') }
  kBucket.add(contact)
  try {
    kBucket._update(contact, 1)
  } catch (exception) {
    test.ok(true)
  }
  test.done()
}

test['deprecated vectorClock results in contact drop'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a'), vectorClock: 3 }
  kBucket.add(contact)
  kBucket._update({ id: new Buffer('a'), vectorClock: 2 }, 0)
  test.equal(kBucket.bucket[0].vectorClock, 3)
  test.done()
}

test['equal vectorClock results in contact marked as most recent'] = function (test) {
  test.expect(1)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a'), vectorClock: 3 }
  kBucket.add(contact)
  kBucket.add({ id: new Buffer('b') })
  kBucket._update(contact, 0)
  test.equal(kBucket.bucket[1], contact)
  test.done()
}

test['more recent vectorClock results in contact update and contact being marked as most recent'] = function (test) {
  test.expect(4)
  var kBucket = new KBucket()
  var contact = { id: new Buffer('a'), old: 'property', vectorClock: 3 }
  kBucket.add(contact)
  kBucket.add({ id: new Buffer('b') })
  kBucket._update({ id: new Buffer('a'), newer: 'property', vectorClock: 4 }, 0)
  test.ok(bufferEqual(kBucket.bucket[1].id, contact.id))
  test.equal(kBucket.bucket[1].vectorClock, 4)
  test.strictEqual(kBucket.bucket[1].old, undefined)
  test.strictEqual(kBucket.bucket[1].newer, 'property')
  test.done()
}

test['should generate "updated"'] = function (test) {
  test.expect(2)
  var kBucket = new KBucket()
  var contact1 = { id: new Buffer('a'), vectorClock: 1 }
  var contact2 = { id: new Buffer('a'), vectorClock: 2 }
  kBucket.on('updated', function (oldContact, newContact) {
    test.deepEqual(oldContact, contact1)
    test.deepEqual(newContact, contact2)
  })
  kBucket.add(contact1)
  kBucket.add(contact2)
  test.done()
}

test['should generate event "updated" when updating a split bucket'] = function (test) {
  test.expect(3)
  var kBucket = new KBucket({
    localNodeId: new Buffer('') // need non-random localNodeId for deterministic splits
  })
  for (var i = 0; i < kBucket.numberOfNodesPerKBucket + 1; ++i) {
    kBucket.add({ id: new Buffer('' + i) })
  }
  test.ok(!kBucket.bucket)
  var contact1 = { id: new Buffer('a'), vectorClock: 1 }
  var contact2 = { id: new Buffer('a'), vectorClock: 2 }
  kBucket.on('updated', function (oldContact, newContact) {
    test.deepEqual(oldContact, contact1)
    test.deepEqual(newContact, contact2)
  })
  kBucket.add(contact1)
  kBucket.add(contact2)
  test.done()
}
